@@ -1,4 +1,4 @@
-# $Id: cgi.py,v 1.31 2004/03/24 12:14:31 jribbens Exp $
+# $Id: cgi.py,v 49de96592142 2011/03/22 23:25:06 jon $
 
 import sys, re, os, Cookie, errno
 try:
@@ -38,13 +38,13 @@ def html_encode(raw):
   Specifically, the following characters are encoded as entities:
     & < > " ' +
   """
-  if not isinstance(raw, unicode):
+  if not isinstance(raw, (str, unicode)):
     raw = str(raw)
   return re.sub(_html_encre, lambda m: _html_encodes[m.group(0)], raw)
 
 def url_encode(raw):
   """Return the string parameter URL-encoded."""
-  if not isinstance(raw, unicode):
+  if not isinstance(raw, (str, unicode)):
     raw = str(raw)
   return re.sub(_url_encre, lambda m: "%%%02X" % ord(m.group(0)), raw)
 
@@ -56,187 +56,21 @@ def url_decode(enc):
 
 __UNDEF__ = []
 
-def _lookup(name, frame, locls):
-  if name in locls:
-    return "local", locls[name]
-  if name in frame.f_globals:
-    return "global", frame.f_globals[name]
-  if "__builtins__" in frame.f_globals and \
-    hasattr(frame.f_globals["__builtins__"], name):
-    return "builtin", getattr(frame.f_globals["__builtins__"], name)
-  return None, __UNDEF__
-
-
-def _scanvars(reader, frame, locls):
-  import tokenize, keyword
-  vrs = []
-  lasttoken = None
-  parent = None
-  prefix = ""
-  for ttype, token, start, end, line in tokenize.generate_tokens(reader):
-    if ttype == tokenize.NEWLINE:
-      break
-    elif ttype == tokenize.NAME and token not in keyword.kwlist:
-      if lasttoken == ".":
-        if parent is not __UNDEF__:
-          value = getattr(parent, token, __UNDEF__)
-          vrs.append((prefix + token, prefix, value))
-      else:
-        (where, value) = _lookup(token, frame, locls)
-        vrs.append((token, where, value))
-    elif token == ".":
-      prefix += lasttoken + "."
-      parent = value
-    else:
-      parent = None
-      prefix = ""
-    lasttoken = token
-  return vrs
-
-
-def _tb_encode(s):
-  return html_encode(s).replace(" ", "&nbsp;")
-
 
 def traceback(req, html=0):
-  import traceback, time, types, linecache, inspect, repr
-  repr = repr.Repr()
-  repr.maxdict = 10
-  repr.maxlist = 10
-  repr.maxtuple = 10
-  repr.maxother = 200
-  repr.maxstring = 200
-  repr = repr.repr
-  (etype, evalue, etb) = sys.exc_info()
-  if type(etype) is types.ClassType:
-    etype = etype.__name__
+  exc = sys.exc_info()
+  import cgitb
   if html:
     try:
       req.clear_headers()
       req.clear_output()
       req.set_header("Content-Type", "text/html; charset=iso-8859-1")
     except SequencingError:
-      req.write("</font></font></font></script></object></blockquote></pre>"
-        "</table></table></table></table></table></table></font></font>")
-    req.write("""\
-<?xml version="1.0" encoding="iso-8859-1"?>
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "DTD/xhtml1-strict.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml">
-<head><title>jonpy traceback: %s</title>
-<style type="text/css"><!--
-BODY { background-color: #f0f0f8; font-family: helveta, arial, sans-serif }
-.tb_head { background-color: #6622aa; color: #ffffff }
-.tb_title { font-size: x-large }
-.tb_frame { background-color: #d8bbff }
-.tb_lineno { font-size: smaller }
-.tb_codehigh { background-color: #ffccee }
-.tb_code { color: #909090 }
-.tb_dump { color: #909090; font-size: smaller }
---></style>
-</head><body>
-<table width="100%%" cellspacing="0" cellpadding="0" border="0">
-<tr class="tb_head">
-<td valign="bottom" class="tb_title">&nbsp;<br /><strong>%s</strong></td>
-<td align="right" valign="bottom">%s<br />%s</td></tr></table>
-<p>A problem occurred in a Python script. Here is the sequence of
-function calls leading up to the error, with the most recent first.</p>
-""" % (_tb_encode(etype), _tb_encode(etype),
-      "Python %s: %s" % (sys.version.split()[0], sys.executable),
-      time.ctime(time.time())))
-  req.error("jonpy error: %s at %s\n" % (etype, time.ctime(time.time())))
-  # this code adapted from the standard cgitb module
-  # unfortunately we cannot use that module directly,
-  # mainly because it won't allow us to output to the log
-  if html:
-    req.write("<p><strong>%s</strong>: %s" % (_tb_encode(etype),
-      _tb_encode(evalue)))
-  req.error("%s: %s\n" % (etype, evalue))
-  #if type(evalue) is types.InstanceType:
-  #  for name in dir(evalue):
-  #    if html:
-  #      req.write("\n<br /><tt>&nbsp;&nbsp;&nbsp;&nbsp;</tt>%s&nbsp;= %s" %
-  #        (_tb_encode(name), _tb_encode(repr(getattr(evalue, name)))))
-  #    req.error("  %s = %s\n" % (name, repr(getattr(evalue, name))))
-  if html:
-    req.write("</p>\n")
-  frames = []
-  records = inspect.getinnerframes(etb, 7)
-  records.reverse()
-  for frame, fn, lnum, func, lines, index in records:
-    if html:
-      req.write("""\
-<table width="100%" cellspacing="0" cellpadding="0" border="0">""")
-    fn = fn and os.path.abspath(fn) or "?"
-    args, varargs, varkw, locls = inspect.getargvalues(frame)
-    if func != "?":
-      fav = inspect.formatargvalues(args, varargs, varkw, locls)
-      if html:
-        req.write("<tr><td class=\"tb_frame\">%s in <strong>%s</strong>%s</td>"
-          "</tr>\n" % (_tb_encode(fn), _tb_encode(func), _tb_encode(fav)))
-      req.error("%s in %s%s\n" % (fn, func, fav))
-    else:
-      if html:
-        req.write("<tr><td class=\"tb_head\">%s</td></tr>\n" %
-          (_tb_encode(fn),))
-      req.error("%s\n" % (fn,))
-    highlight = {}
-    def reader(lnum=[lnum]):
-      highlight[lnum[0]] = 1
-      try:
-        return linecache.getline(fn, lnum[0])
-      finally:
-        lnum[0] += 1
-    vrs = _scanvars(reader, frame, locls)
-    if index is not None:
-      i = lnum - index
-      for line in lines:
-        if html:
-          if i in highlight:
-            style = "tb_codehigh"
-          else:
-            style = "tb_code"
-          req.write("<tr><td class=\"%s\"><code><span class=\"tb_lineno\">"
-            "%s</span>&nbsp;%s</code></td></tr>\n" %
-            (style, "&nbsp;" * (5 - len(str(i))) + str(i), _tb_encode(line)))
-        req.error("%s %s" % (" " * (5-len(str(i))) + str(i), line))
-        i += 1
-    done = {}
-    dump = []
-    htdump = []
-    for name, where, value in vrs:
-      if name in done:
-        continue
-      done[name] = 1
-      if value is not __UNDEF__:
-        if where == "global":
-          dump.append("global %s = %s" % (name, repr(value)))
-          htdump.append("<em>global</em> <strong>%s</strong>&nbsp;= %s" %
-            (_tb_encode(name), _tb_encode(repr(value))))
-        elif where == "builtin":
-          dump.append("builtin %s = %s" % (name, repr(value)))
-          htdump.append("<em>builtin</em> <strong>%s</strong>&nbsp;= %s" %
-            (_tb_encode(name), _tb_encode(repr(value))))
-        elif where == "local":
-          dump.append("%s = %s" % (name, repr(value)))
-          htdump.append("<strong>%s</strong>&nbsp;= %s" %
-            (_tb_encode(name), _tb_encode(repr(value))))
-        else:
-          dump.append("%s%s = %s" % (where, name.split(".")[-1], repr(value)))
-          htdump.append("%s<strong>%s</strong>&nbsp;= %s" %
-            (_tb_encode(where), _tb_encode(name.split(".")[-1]),
-            _tb_encode(repr(value))))
-      else:
-        dump.append("%s undefined" % (name,))
-        htdump.append("%s <em>undefined</em>" % (_tb_encode(name,)))
-    if html:
-      req.write("<tr><td class=\"tb_dump\">%s</td></tr>\n" %
-        (", ".join(htdump),))
-    req.error(", ".join(dump) + "\n")
-    if html:
-      req.write("</table>\n")
-  if html:
-    req.write("</body></html>\n")
-  linecache.clearcache()
+      pass
+    cgitb.Hook(file=req)(*exc)
+  s = StringIO.StringIO()
+  cgitb.Hook(file=s, format="text")(*exc)
+  req.error(s.getvalue())
 
 
 class Request(object):
@@ -258,12 +92,13 @@ class Request(object):
     self._output = StringIO.StringIO()
     self._pos = 0
     self.closed = 0
+    self._encoding = self._inputencoding = None
     try:
       del self.params
     except AttributeError:
       pass
     self.cookies = Cookie.SimpleCookie()
-    if self.environ.has_key("HTTP_COOKIE"):
+    if "HTTP_COOKIE" in self.environ:
       self.cookies.load(self.environ["HTTP_COOKIE"])
     self.aborted = 0
     self.set_header("Content-Type", "text/html; charset=iso-8859-1")
@@ -273,8 +108,8 @@ class Request(object):
       self.params = {}
       self._read_cgi_data(self.environ, self.stdin)
       return self.__dict__["params"]
-    raise AttributeError, "%s instance has no attribute %s" % \
-      (self.__class__.__name__, `name`)
+    raise AttributeError("%s instance has no attribute %s" %
+      (self.__class__.__name__, repr(name)))
 
   def close(self):
     """Closes the output stream."""
@@ -285,39 +120,38 @@ class Request(object):
 
   def _check_open(self):
     if self.closed:
-      raise ValueError, "I/O operation on closed file"
+      raise ValueError("I/O operation on closed file")
 
   def output_headers(self):
     """Output the list of headers."""
     self._check_open()
     if self._doneHeaders:
-      raise SequencingError, "output_headers() called twice"
-    for pair in self._headers:
-      self._write("%s: %s\r\n" % pair)
-    self._write("\r\n")
+      raise SequencingError("output_headers() called twice")
+    self._write("".join(["%s: %s\r\n" % pair for pair in self._headers])
+      + "\r\n")
     self._doneHeaders = 1
 
   def clear_headers(self):
     """Clear the list of headers."""
     self._check_open()
     if self._doneHeaders:
-      raise SequencingError, "cannot clear_headers() after output_headers()"
+      raise SequencingError("cannot clear_headers() after output_headers()")
     self._headers = []
 
   def add_header(self, hdr, val):
     """Add a header to the list of headers."""
     self._check_open()
     if self._doneHeaders:
-      raise SequencingError, \
-        "cannot add_header(%s) after output_headers()" % `hdr`
+      raise SequencingError(
+        "cannot add_header(%s) after output_headers()" % repr(hdr))
     self._headers.append((hdr, val))
 
   def set_header(self, hdr, val):
     """Add a header to the list of headers, replacing any existing values."""
     self._check_open()
     if self._doneHeaders:
-      raise SequencingError, \
-        "cannot set_header(%s) after output_headers()" % `hdr`
+      raise SequencingError(
+        "cannot set_header(%s) after output_headers()" % repr(hdr))
     self.del_header(hdr)
     self._headers.append((hdr, val))
 
@@ -336,8 +170,8 @@ class Request(object):
     """Removes all values for a header from the list of headers."""
     self._check_open()
     if self._doneHeaders:
-      raise SequencingError, \
-        "cannot del_header(%s) after output_headers()" % `hdr`
+      raise SequencingError(
+        "cannot del_header(%s) after output_headers()" % repr(hdr))
     hdr = hdr.lower()
     while 1:
       for s in self._headers:
@@ -347,6 +181,20 @@ class Request(object):
       else:
         break
 
+  def append_header_value(self, hdr, val):
+    """Adds a value to a header that contains a comma-separated list of values.
+    If the value is already in the list, it is not added again. If the header
+    does not exist, it is created containing the single value specified."""
+    current = self.get_header(hdr)
+    if not current:
+      self.set_header(hdr, val)
+      return val
+    if val.lower() in [x.strip().lower() for x in current.split(",")]:
+      return current
+    val = current + "," + val
+    self.set_header(hdr, val)
+    return val
+
   def set_buffering(self, f):
     """Specifies whether or not body output is buffered."""
     self._check_open()
@@ -354,6 +202,14 @@ class Request(object):
       self.flush()
     self._bufferOutput = f
 
+  def set_encoding(self, encoding, inputencoding=__UNDEF__):
+    self._encoding = encoding
+    if inputencoding is not __UNDEF__:
+      self._inputencoding = inputencoding
+
+  def get_encoding(self):
+    return self._encoding
+
   def flush(self):
     """Flushes the body output."""
     self._check_open()
@@ -369,7 +225,7 @@ class Request(object):
     """Discards the contents of the body output buffer."""
     self._check_open()
     if not self._bufferOutput:
-      raise SequencingError, "cannot clear output when not buffering"
+      raise SequencingError("cannot clear output when not buffering")
     self._output.seek(0, 0)
     self._output.truncate()
 
@@ -379,12 +235,12 @@ class Request(object):
     go to the client.
     
     Must be overridden by the sub-class."""
-    raise NotImplementedError, "error must be overridden"
+    raise NotImplementedError("error must be overridden")
 
   def _write(self, s):
     """Sends some data to the client."""
     """Must be overridden by the sub-class."""
-    raise NotImplementedError, "_write must be overridden"
+    raise NotImplementedError("_write must be overridden")
 
   def _flush(self):
     """Flushes data to the client."""
@@ -399,7 +255,15 @@ class Request(object):
   def write(self, s):
     """Sends some data to the client."""
     self._check_open()
-    s = str(s)
+    if self._encoding:
+      if not isinstance(s, unicode):
+        if self._inputencoding:
+          s = unicode(s, self._inputencoding)
+        else:
+          s = unicode(s)
+      s = s.encode(self._encoding)
+    else:
+      s = str(s)
     if self._bufferOutput:
       self._output.write(s)
     else:
@@ -422,11 +286,11 @@ class Request(object):
     elif whence == 2:
       newpos = currentlen + offset
     else:
-      raise ValueError, "Bad 'whence' argument to seek()"
+      raise ValueError("Bad 'whence' argument to seek()")
     if newpos == currentpos:
       return
     elif newpos < self._pos:
-      raise ValueError, "Cannot seek backwards into already-sent data"
+      raise ValueError("Cannot seek backwards into already-sent data")
     elif newpos <= currentlen:
       self._output.seek(newpos - self._pos)
     else:
@@ -456,7 +320,7 @@ class Request(object):
       if name.endswith("!") or name.endswith("!*"):
         continue
       if name.endswith("*"):
-        if self.params.has_key(name):
+        if name in self.params:
           self.params[name].append(val)
         else:
           self.params[name] = [val]
@@ -481,7 +345,7 @@ class Request(object):
         continue
       name = entity.content_disposition[1].get("name")
       if name[-1:] == "*":
-        if self.params.has_key(name):
+        if name in self.params:
           if name[-2:-1] == "!":
             self.params[name].append(entity)
           else:
@@ -498,7 +362,7 @@ class Request(object):
 
   def _read_cgi_data(self, environ, inf):
     """Read input data from the client and set up the object attributes."""
-    if environ.has_key("QUERY_STRING"):
+    if "QUERY_STRING" in environ:
       self._mergevars(environ["QUERY_STRING"])
     if environ.get("REQUEST_METHOD") == "POST":
       if environ.get("CONTENT_TYPE", "").startswith("multipart/form-data"):
@@ -522,43 +386,47 @@ class Request(object):
 
 
 class GZipMixIn(object):
-  def _init(self):
+  def _init(self, *args, **kwargs):
     self._gzip = None
     self._gzip_level = 6
-    super(GZipMixIn, self)._init()
+    super(GZipMixIn, self)._init(*args, **kwargs)
 
   def _close(self):
+    parent = super(GZipMixIn, self)
     if self._gzip:
       import struct
-      super(GZipMixIn, self)._write(self._gzip.flush(self._gzip_zlib.Z_FINISH))
-      super(GZipMixIn, self)._write(
-        struct.pack("<II", self._gzip_crc, self._gzip_length))
-      super(GZipMixIn, self)._flush()
+      parent._write(self._gzip.flush(self._gzip_zlib.Z_FINISH))
+      parent._write(
+        struct.pack("<II", self._gzip_crc & 0xffffffff, self._gzip_length))
+      parent._flush()
       self._gzip = None
-    super(GZipMixIn, self)._close()
+    parent._close()
 
   def gzip_level(self, level=6):
     """Enable/disable gzip output compression."""
     if self._gzip_level == level:
       return
     if self._doneHeaders:
-      raise SequencingError, "Cannot adjust compression - headers already sent"
+      raise SequencingError("Cannot adjust compression - headers already sent")
     self._gzip_level = level
 
   def _write(self, s):
+    parent = super(GZipMixIn, self)
     if not self._gzip:
-      super(GZipMixIn, self)._write(s)
+      parent._write(s)
       return
     self._gzip_crc = self._gzip_zlib.crc32(s, self._gzip_crc)
     self._gzip_length += len(s)
-    super(GZipMixIn, self)._write(self._gzip.compress(s))
+    parent._write(self._gzip.compress(s))
 
   def output_headers(self):
+    parent = super(GZipMixIn, self)
     if self._gzip_level == 0:
-      super(GZipMixIn, self).output_headers()
+      parent.output_headers()
       return
+    self.append_header_value("Vary", "Accept-Encoding")
     gzip_ok = 0
-    if self.environ.has_key("HTTP_ACCEPT_ENCODING"):
+    if "HTTP_ACCEPT_ENCODING" in self.environ:
       encodings = [[a.strip() for a in x.split(";", 1)]
         for x in self.environ["HTTP_ACCEPT_ENCODING"].split(",")]
       for encoding in encodings:
@@ -574,30 +442,23 @@ class GZipMixIn(object):
     if gzip_ok:
       try:
         import zlib
-        encoding = "gzip"
-        encoding = self.get_header("Content-Encoding")
-        if encoding is None:
-          encoding = "gzip"
-        else:
-          encoding += ", gzip"
-        self.set_header("Content-Encoding", encoding)
+        self.append_header_value("Content-Encoding", "gzip")
         self.del_header("Content-Length")
-        super(GZipMixIn, self).output_headers()
+        parent.output_headers()
         self._gzip = zlib.compressobj(self._gzip_level, 8, -15)
         self._gzip_zlib = zlib
         self._gzip_crc = self._gzip_length = 0
-        super(GZipMixIn, self)._write(
-          "\x1f\x8b\x08\x00\x00\x00\x00\x00\x00\x03")
+        parent._write("\x1f\x8b\x08\x00\x00\x00\x00\x00\x00\x03")
         return
       except ImportError:
         pass
-    super(GZipMixIn, self).output_headers()
+    parent.output_headers()
 
   def _flush(self):
+    parent = super(GZipMixIn, self)
     if self._gzip:
-      super(GZipMixIn, self)._write(
-        self._gzip.flush(self._gzip_zlib.Z_SYNC_FLUSH))
-    super(GZipMixIn, self)._flush()
+      parent._write(self._gzip.flush(self._gzip_zlib.Z_SYNC_FLUSH))
+    parent._flush()
 
 
 class CGIRequest(Request):
@@ -627,6 +488,9 @@ class CGIRequest(Request):
   def error(self, s):
     self.__err.write(s)
 
+  def _close(self):
+    self.__out.close()
+
   def _write(self, s):
     if not self.aborted:
       try:
@@ -656,7 +520,7 @@ class Handler(object):
   """Handle a request."""
   def process(self, req):
     """Handle a request. req is a Request object."""
-    raise NotImplementedError, "handler process function must be overridden"
+    raise NotImplementedError("handler process function must be overridden")
 
   def traceback(self, req):
     """Display a traceback, req is a Request object."""
